package com.bipt.intelligentapplicationorchestrationservice.service.Impl;

import com.bipt.intelligentapplicationorchestrationservice.constant.MessageConstant;
import com.bipt.intelligentapplicationorchestrationservice.enumeration.ModelLifecycle;
import com.bipt.intelligentapplicationorchestrationservice.mapper.ModelMapper;
import com.bipt.intelligentapplicationorchestrationservice.pojo.*;
import com.bipt.intelligentapplicationorchestrationservice.service.ModelService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.List;

@Slf4j
@Service
public class ModelServiceImpl implements ModelService {
    @Autowired
    private ModelMapper modelMapper;
    /**
     * 创建模型
     * @param dto
     */
    @Transactional  //事务注解
    @Override
    public void createModel(ModelDTO dto) {
        log.info("创建模型");
        //1.插入模型基本信息
        ModelInfo modelInfo = new ModelInfo();
        BeanUtils.copyProperties(dto, modelInfo, "id");
        modelMapper.insertModelInfo(modelInfo);
        log.info("模型id: {}",  modelInfo.getId());
        //2.插入模型版本信息
        ModelVersion modelVersion = new ModelVersion();
        //使用工具类将dto属性复制给modelVersion
        BeanUtils.copyProperties(dto, modelVersion, "id");
        modelVersion.setModelId(modelInfo.getId());
        modelVersion.setCreateTime(LocalDateTime.now());
        modelVersion.setUpdateTime(LocalDateTime.now());
        modelVersion.setOperateUser("zs");
        modelMapper.insertModelVersion(modelVersion);
    }

    /**
     * 新增模型版本
     * @param dto
     */
    @Override
    public void createModelVersion(ModelDTO dto) {
        log.info("新增模型版本");
        ModelVersion modelVersion = new ModelVersion();
        BeanUtils.copyProperties(dto, modelVersion, "id");
        modelVersion.setModelId(dto.getId());
        modelVersion.setCreateTime(LocalDateTime.now());
        modelVersion.setUpdateTime(LocalDateTime.now());
        modelVersion.setOperateUser("zs");
        modelMapper.insertModelVersion(modelVersion);
    }

    /**
     * 查询模型列表
     */
    @Override
    public List<ModelVO> list() {
        log.info("查询模型列表");
        List<ModelVO> modelVOList = modelMapper.list();
        return modelVOList;
    }

    @Override
    public ModelVersion detail(Long id) {
        log.info("查询模型详情");
        ModelVersion modelVersion = modelMapper.selectById(id);
        return modelVersion;
    }

    @Override
    public void updateModel(ModelVersionDTO dto) {
        log.info("更新模型");
        modelMapper.update(dto);
    }

    @Override
    public void deleteModelVersion(Long id) {
        log.info("删除模型版本");
        modelMapper.deleteModelVersion(id);
    }

    /**
     * 更新模型生命周期
     * @param id   版本表id
     * @param
     */
    @Override
    public void updateLifeCycle(Long id, String lifeCycleDescription) {
        log.info("更新模型生命周期");

        // 1. 查询当前生命周期并转为枚举
        String currentLifeCycleStr = modelMapper.selectLifeCycleById(id);
        if (currentLifeCycleStr == null) {
            log.error("未找到对应模型信息，无法更新生命周期");
            throw new IllegalArgumentException(MessageConstant.LifeCycle_No_Exist);
        }

        ModelLifecycle currentLifeCycle;
        ModelLifecycle targetLifeCycle;
        try {
            currentLifeCycle = ModelLifecycle.valueOf(currentLifeCycleStr.trim()); // 数据库中是英文
            targetLifeCycle = ModelLifecycle.fromDescription((lifeCycleDescription).trim());      // 前端传中文
        } catch (IllegalArgumentException e) {
            log.error(MessageConstant.LifeCycle_Undefined + ":{}", e.getMessage());
            throw e;
        }

        // 2. 业务逻辑校验
        switch (currentLifeCycle) {
            case DEPLOYED:
                if (targetLifeCycle == ModelLifecycle.DESIGNING) {
                    log.error("已部署的模型不能直接调整成设计，需先下线再设计");
                    throw new IllegalArgumentException(MessageConstant.ERROR_DEPLOYED_TO_DESIGNING);
                }
                break;
            case ABANDONED:
                log.error("已废弃的模型只能查看信息，不能更新生命周期");
                throw new IllegalArgumentException(MessageConstant.ERROR_ABANDONED_CANNOT_UPDATE);
            case TRAINING:
                if (targetLifeCycle != ModelLifecycle.DESIGNING && targetLifeCycle != ModelLifecycle.EVALUATING) {
                    log.error("训练中的模型只能调整成设计和评估");
                    throw new IllegalArgumentException(MessageConstant.ERROR_TRAINING_INVALID_TRANSITION);
                }
                break;
            default:
                break;
        }

        // 3. 执行更新
        int affectedRows = modelMapper.updateLifeCycleById(id, targetLifeCycle.getDbValue());
        if (affectedRows == 0) {
            log.error("更新模型生命周期失败");
            throw new RuntimeException(MessageConstant.UPDATE_FAILURE);
        }

        log.info("模型生命周期更新成功，新状态为: {}", targetLifeCycle);
    }

}
